@@ -29,6 +29,7 @@
 #include "tkrzw_dbm_shard.h"
 #include "tkrzw_file.h"
 #include "tkrzw_file_poly.h"
+#include "tkrzw_index.h"
 #include "tkrzw_langc.h"
 #include "tkrzw_lib_common.h"
 #include "tkrzw_str_util.h"
@@ -422,7 +423,7 @@ TkrzwKeyValuePair* tkrzw_future_get_str_pair(TkrzwFuture* future) {
 }
 
 TkrzwStr* tkrzw_future_get_str_array(TkrzwFuture* future, int32_t* num_elems) {
-  assert(future != nullptr);
+  assert(future != nullptr && num_elems != nullptr);
   try {
     StatusFuture* xfuture = reinterpret_cast<StatusFuture*>(future);
     const auto& result = xfuture->GetStringVector();
@@ -445,7 +446,7 @@ TkrzwStr* tkrzw_future_get_str_array(TkrzwFuture* future, int32_t* num_elems) {
 }
 
 TkrzwKeyValuePair* tkrzw_future_get_str_map(TkrzwFuture* future, int32_t* num_elems) {
-  assert(future != nullptr);
+  assert(future != nullptr && num_elems != nullptr);
   try {
     StatusFuture* xfuture = reinterpret_cast<StatusFuture*>(future);
     const auto& result = xfuture->GetStringMap();
@@ -535,7 +536,7 @@ bool tkrzw_dbm_close(TkrzwDBM* dbm) {
   try {
     ParamDBM* xdbm = reinterpret_cast<ParamDBM*>(dbm);
     last_status = xdbm->Close();
-    bool rv = last_status == Status::SUCCESS;
+    const bool rv = last_status == Status::SUCCESS;
     delete xdbm;
     return rv;
   } catch (const std::exception& e) {
@@ -2261,7 +2262,7 @@ bool tkrzw_file_close(TkrzwFile* file) {
   try {
     PolyFile* xfile = reinterpret_cast<PolyFile*>(file);
     last_status = xfile->Close();
-    bool rv = last_status == Status::SUCCESS;
+    const bool rv = last_status == Status::SUCCESS;
     delete xfile;
     return rv;
   } catch (const std::exception& e) {
@@ -2394,6 +2395,352 @@ TkrzwStr* tkrzw_file_search(
   }
 }
 
+TkrzwIndex* tkrzw_index_open(const char* path, bool writable, const char* params) {
+  assert(path != nullptr && params != nullptr);
+  try {
+    std::map<std::string, std::string> xparams = StrSplitIntoMap(params, ",", "=");
+    int32_t open_options = 0;
+    if (tkrzw::StrToBool(tkrzw::SearchMap(xparams, "truncate", "false"))) {
+      open_options |= tkrzw::File::OPEN_TRUNCATE;
+    }
+    if (tkrzw::StrToBool(tkrzw::SearchMap(xparams, "no_create", "false"))) {
+      open_options |= tkrzw::File::OPEN_NO_CREATE;
+    }
+    if (tkrzw::StrToBool(tkrzw::SearchMap(xparams, "no_wait", "false"))) {
+      open_options |= tkrzw::File::OPEN_NO_WAIT;
+    }
+    if (tkrzw::StrToBool(tkrzw::SearchMap(xparams, "no_lock", "false"))) {
+      open_options |= tkrzw::File::OPEN_NO_LOCK;
+    }
+    if (tkrzw::StrToBool(tkrzw::SearchMap(xparams, "sync_hard", "false"))) {
+      open_options |= tkrzw::File::OPEN_SYNC_HARD;
+    }
+    xparams.erase("truncate");
+    xparams.erase("no_create");
+    xparams.erase("no_wait");
+    xparams.erase("no_lock");
+    xparams.erase("sync_hard");
+    PolyIndex* index = new tkrzw::PolyIndex();
+    last_status = index->Open(path, writable, open_options, xparams);
+    if (last_status != Status::SUCCESS) {
+      delete index;
+      return nullptr;
+    }
+    return reinterpret_cast<TkrzwIndex*>(index);
+  } catch (const std::exception& e) {
+    tkrzw_set_last_status(TKRZW_STATUS_SYSTEM_ERROR, nullptr);
+    return nullptr;
+  }
+}
+
+bool tkrzw_index_close(TkrzwIndex* index) {
+  assert(index != nullptr);
+  try {
+    PolyIndex* xindex = reinterpret_cast<PolyIndex*>(index);
+    last_status = xindex->Close();
+    const bool rv = last_status == Status::SUCCESS;
+    delete xindex;
+    return rv;
+  } catch (const std::exception& e) {
+    tkrzw_set_last_status(TKRZW_STATUS_SYSTEM_ERROR, nullptr);
+    return false;
+  }
+}
+
+bool tkrzw_index_check(
+    TkrzwIndex* index, const char* key_ptr, int32_t key_size,
+    const char* value_ptr, int32_t value_size) {
+  assert(index != nullptr && key_ptr != nullptr && value_ptr != nullptr);
+  try {
+    if (key_size < 0) {
+      key_size = std::strlen(key_ptr);
+    }
+    if (value_size < 0) {
+      value_size = std::strlen(value_ptr);
+    }
+    PolyIndex* xindex = reinterpret_cast<PolyIndex*>(index);
+    return xindex->Check(std::string_view(key_ptr, key_size),
+                         std::string_view(value_ptr, value_size));
+  } catch (const std::exception& e) {
+    tkrzw_set_last_status(TKRZW_STATUS_SYSTEM_ERROR, nullptr);
+    return false;
+  }
+}
+
+TkrzwStr* tkrzw_index_get_values(
+    TkrzwIndex* index, const char* key_ptr, int32_t key_size, int32_t max, int32_t* num_elems) {
+  assert(index != nullptr && key_ptr != nullptr && num_elems != nullptr);
+  try {
+    if (key_size < 0) {
+      key_size = std::strlen(key_ptr);
+    }
+    PolyIndex* xindex = reinterpret_cast<PolyIndex*>(index);
+    const auto& values = xindex->GetValues(std::string_view(key_ptr, key_size), max);
+    TkrzwStr* array = static_cast<TkrzwStr*>(xmalloc(sizeof(TkrzwStr) * values.size() + 1));
+    for (size_t i = 0; i < values.size(); i++) {
+      const auto& xelem = values[i];
+      auto& elem = array[i];
+      char* ptr = static_cast<char*>(xmalloc(xelem.size() + 1));
+      std::memcpy(ptr, xelem.c_str(), xelem.size() + 1);
+      elem.ptr = ptr;
+      elem.size = xelem.size();
+    }
+    *num_elems = values.size();
+    return array;
+  } catch (const std::exception& e) {
+    tkrzw_set_last_status(TKRZW_STATUS_SYSTEM_ERROR, nullptr);
+    TkrzwStr* array = static_cast<TkrzwStr*>(xmalloc(1));
+    *num_elems = 0;
+    return array;
+  }
+}
+
+bool tkrzw_index_add(
+    TkrzwIndex* index, const char* key_ptr, int32_t key_size,
+    const char* value_ptr, int32_t value_size) {
+  assert(index != nullptr && key_ptr != nullptr && value_ptr != nullptr);
+  try {
+    if (key_size < 0) {
+      key_size = std::strlen(key_ptr);
+    }
+    if (value_size < 0) {
+      value_size = std::strlen(value_ptr);
+    }
+    PolyIndex* xindex = reinterpret_cast<PolyIndex*>(index);
+    last_status = xindex->Add(std::string_view(key_ptr, key_size),
+                              std::string_view(value_ptr, value_size));
+    return last_status == Status::SUCCESS;
+  } catch (const std::exception& e) {
+    tkrzw_set_last_status(TKRZW_STATUS_SYSTEM_ERROR, nullptr);
+    return false;
+  }
+}
+
+bool tkrzw_index_remove(
+    TkrzwIndex* index, const char* key_ptr, int32_t key_size,
+    const char* value_ptr, int32_t value_size) {
+  assert(index != nullptr && key_ptr != nullptr && value_ptr != nullptr);
+  try {
+    if (key_size < 0) {
+      key_size = std::strlen(key_ptr);
+    }
+    if (value_size < 0) {
+      value_size = std::strlen(value_ptr);
+    }
+    PolyIndex* xindex = reinterpret_cast<PolyIndex*>(index);
+    last_status = xindex->Remove(std::string_view(key_ptr, key_size),
+                                 std::string_view(value_ptr, value_size));
+    return last_status == Status::SUCCESS;
+  } catch (const std::exception& e) {
+    tkrzw_set_last_status(TKRZW_STATUS_SYSTEM_ERROR, nullptr);
+    return false;
+  }
+}
+
+int32_t tkrzw_index_count(TkrzwIndex* index) {
+  assert(index != nullptr);
+  try {
+    PolyIndex* xindex = reinterpret_cast<PolyIndex*>(index);
+    return xindex->Count();
+  } catch (const std::exception& e) {
+    tkrzw_set_last_status(TKRZW_STATUS_SYSTEM_ERROR, nullptr);
+    return -1;
+  }
+}
+
+char* tkrzw_index_get_file_path(TkrzwIndex* index) {
+  assert(index != nullptr);
+  try {
+    PolyIndex* xindex = reinterpret_cast<PolyIndex*>(index);
+    std::string path = xindex->GetFilePath();
+    char* path_ptr = reinterpret_cast<char*>(xmalloc(path.size() + 1));
+    std::memcpy(path_ptr, path.c_str(), path.size() + 1);
+    return path_ptr;
+  } catch (const std::exception& e) {
+    tkrzw_set_last_status(TKRZW_STATUS_SYSTEM_ERROR, nullptr);
+    char* path_ptr = reinterpret_cast<char*>(xmalloc(1));
+    *path_ptr = '\0';
+    return path_ptr;
+  }
+}
+
+bool tkrzw_index_clear(TkrzwIndex* index) {
+  assert(index != nullptr);
+  try {
+    PolyIndex* xindex = reinterpret_cast<PolyIndex*>(index);
+    last_status = xindex->Clear();
+    return last_status == Status::SUCCESS;
+  } catch (const std::exception& e) {
+    tkrzw_set_last_status(TKRZW_STATUS_SYSTEM_ERROR, nullptr);
+    return false;
+  }
+}
+
+bool tkrzw_index_rebuild(TkrzwIndex* index) {
+  assert(index != nullptr);
+  try {
+    PolyIndex* xindex = reinterpret_cast<PolyIndex*>(index);
+    last_status = xindex->Rebuild();
+    return last_status == Status::SUCCESS;
+  } catch (const std::exception& e) {
+    tkrzw_set_last_status(TKRZW_STATUS_SYSTEM_ERROR, nullptr);
+    return false;
+  }
+}
+
+bool tkrzw_index_synchronize(TkrzwIndex* index, bool hard) {
+  assert(index != nullptr);
+  try {
+    PolyIndex* xindex = reinterpret_cast<PolyIndex*>(index);
+    last_status = xindex->Synchronize(hard);
+    return last_status == Status::SUCCESS;
+  } catch (const std::exception& e) {
+    tkrzw_set_last_status(TKRZW_STATUS_SYSTEM_ERROR, nullptr);
+    return false;
+  }
+}
+
+bool tkrzw_index_is_writable(TkrzwIndex* index) {
+  assert(index != nullptr);
+  try {
+    PolyIndex* xindex = reinterpret_cast<PolyIndex*>(index);
+    return xindex->IsWritable();
+  } catch (const std::exception& e) {
+    tkrzw_set_last_status(TKRZW_STATUS_SYSTEM_ERROR, nullptr);
+    return false;
+  }
+}
+
+TkrzwIndexIter* tkrzw_index_make_iterator(TkrzwIndex* index) {
+  assert(index != nullptr);
+  try {
+    PolyIndex* xindex = reinterpret_cast<PolyIndex*>(index);
+    return reinterpret_cast<TkrzwIndexIter*>(xindex->MakeIterator().release());
+  } catch (const std::exception& e) {
+    tkrzw_set_last_status(TKRZW_STATUS_SYSTEM_ERROR, nullptr);
+    return nullptr;
+  }
+}
+
+void tkrzw_index_iter_free(TkrzwIndexIter* iter) {
+  assert(iter != nullptr);
+  try {
+    PolyIndex::Iterator* xiter = reinterpret_cast<PolyIndex::Iterator*>(iter);
+    delete xiter;
+  } catch (const std::exception& e) {
+    tkrzw_set_last_status(TKRZW_STATUS_SYSTEM_ERROR, nullptr);
+  }
+}
+
+void tkrzw_index_iter_first(TkrzwIndexIter* iter) {
+  assert(iter != nullptr);
+  try {
+    PolyIndex::Iterator* xiter = reinterpret_cast<PolyIndex::Iterator*>(iter);
+    xiter->First();
+  } catch (const std::exception& e) {
+    tkrzw_set_last_status(TKRZW_STATUS_SYSTEM_ERROR, nullptr);
+  }
+}
+
+void tkrzw_index_iter_last(TkrzwIndexIter* iter) {
+  assert(iter != nullptr);
+  try {
+    PolyIndex::Iterator* xiter = reinterpret_cast<PolyIndex::Iterator*>(iter);
+    xiter->Last();
+  } catch (const std::exception& e) {
+    tkrzw_set_last_status(TKRZW_STATUS_SYSTEM_ERROR, nullptr);
+  }
+}
+
+void tkrzw_index_iter_jump(TkrzwIndexIter* iter, const char* key_ptr, int32_t key_size,
+                           const char* value_ptr, int32_t value_size) {
+  assert(iter != nullptr && key_ptr != nullptr && value_ptr != nullptr);
+  try {
+    PolyIndex::Iterator* xiter = reinterpret_cast<PolyIndex::Iterator*>(iter);
+    if (key_size < 0) {
+      key_size = std::strlen(key_ptr);
+    }
+    if (value_size < 0) {
+      value_size = std::strlen(value_ptr);
+    }
+    xiter->Jump(std::string_view(key_ptr, key_size), std::string_view(value_ptr, value_size));
+  } catch (const std::exception& e) {
+    tkrzw_set_last_status(TKRZW_STATUS_SYSTEM_ERROR, nullptr);
+  }
+}
+
+void tkrzw_index_iter_next(TkrzwIndexIter* iter) {
+  assert(iter != nullptr);
+  try {
+    PolyIndex::Iterator* xiter = reinterpret_cast<PolyIndex::Iterator*>(iter);
+    xiter->Next();
+  } catch (const std::exception& e) {
+    tkrzw_set_last_status(TKRZW_STATUS_SYSTEM_ERROR, nullptr);
+  }
+}
+
+void tkrzw_index_iter_previous(TkrzwIndexIter* iter) {
+  assert(iter != nullptr);
+  try {
+    PolyIndex::Iterator* xiter = reinterpret_cast<PolyIndex::Iterator*>(iter);
+    xiter->Previous();
+  } catch (const std::exception& e) {
+    tkrzw_set_last_status(TKRZW_STATUS_SYSTEM_ERROR, nullptr);
+  }
+}
+
+bool tkrzw_index_iter_get(
+    TkrzwIndexIter* iter, char** key_ptr, int32_t* key_size,
+    char** value_ptr, int32_t* value_size) {
+  assert(iter != nullptr);
+  try {
+    PolyIndex::Iterator* xiter = reinterpret_cast<PolyIndex::Iterator*>(iter);
+    bool rv = false;
+    if (key_ptr == nullptr && value_ptr == nullptr) {
+      rv = xiter->Get();
+    } else if (value_ptr == nullptr) {
+      std::string key;
+      if (xiter->Get(&key)) {
+        *key_ptr = static_cast<char*>(xmalloc(key.size() + 1));
+        std::memcpy(*key_ptr, key.c_str(), key.size() + 1);
+        if (key_size != nullptr) {
+          *key_size = key.size();
+        }
+        rv = true;
+      }
+    } else if (key_ptr == nullptr) {
+      std::string value;
+      if (xiter->Get(nullptr, &value)) {
+        *value_ptr = static_cast<char*>(xmalloc(value.size() + 1));
+        std::memcpy(*value_ptr, value.c_str(), value.size() + 1);
+        if (value_size != nullptr) {
+          *value_size = value.size();
+        }
+        rv = true;
+      }
+    } else {
+      std::string key, value;
+      if (xiter->Get(&key, &value)) {
+        *key_ptr = static_cast<char*>(xmalloc(key.size() + 1));
+        std::memcpy(*key_ptr, key.c_str(), key.size() + 1);
+        if (key_size != nullptr) {
+          *key_size = key.size();
+        }
+        *value_ptr = static_cast<char*>(xmalloc(value.size() + 1));
+        std::memcpy(*value_ptr, value.c_str(), value.size() + 1);
+        if (value_size != nullptr) {
+          *value_size = value.size();
+        }
+        rv = true;
+      }
+    }
+    return rv;
+  } catch (const std::exception& e) {
+    tkrzw_set_last_status(TKRZW_STATUS_SYSTEM_ERROR, nullptr);
+    return false;
+  }
+}
+
 }  // extern "C"
 
 // END OF FILE
