@@ -1139,4 +1139,111 @@ TEST(LangCTest, File) {
   EXPECT_TRUE(tkrzw_file_close(file));
 }
 
+TEST(LangCTest, Index) {
+  tkrzw::TemporaryDirectory tmp_dir(true, "tkrzw-");
+  const std::string file_path = tmp_dir.MakeUniquePath("casket-", ".tkt");
+  TkrzwIndex* index = tkrzw_index_open(file_path.c_str(), true, "truncate=true,num_buckets=10");
+  ASSERT_NE(nullptr, index);
+  EXPECT_FALSE(tkrzw_index_check(index, "single", -1, "1", -1));
+  EXPECT_FALSE(tkrzw_index_check(index, "double", 5, "11", 2));
+  EXPECT_TRUE(tkrzw_index_add(index, "single", -1, "1", -1));
+  EXPECT_TRUE(tkrzw_index_add(index, "double", 5, "11", 2));
+  EXPECT_TRUE(tkrzw_index_check(index, "single", -1, "1", -1));
+  EXPECT_TRUE(tkrzw_index_check(index, "double", 5, "11", 2));
+  EXPECT_TRUE(tkrzw_index_add(index, "single", -1, "2", -1));
+  EXPECT_TRUE(tkrzw_index_add(index, "double", -1, "22", -1));
+  EXPECT_TRUE(tkrzw_index_add(index, "triple", -1, "222", -1));
+  int32_t num_values = 0;
+  TkrzwStr* values = tkrzw_index_get_values(index, "single", -1, 0, &num_values);
+  ASSERT_NE(nullptr, values);
+  ASSERT_EQ(2, num_values);
+  EXPECT_EQ("1", std::string(values[0].ptr, values[0].size));
+  EXPECT_EQ("2", std::string(values[1].ptr, values[1].size));
+  tkrzw_free_str_array(values, num_values);
+  values = tkrzw_index_get_values(index, "triple", -1, 0, &num_values);
+  ASSERT_NE(nullptr, values);
+  ASSERT_EQ(1, num_values);
+  EXPECT_EQ("222", std::string(values[0].ptr, values[0].size));
+  tkrzw_free_str_array(values, num_values);
+  values = tkrzw_index_get_values(index, "foo", -1, 0, &num_values);
+  ASSERT_NE(nullptr, values);
+  ASSERT_EQ(0, num_values);
+  tkrzw_free_str_array(values, num_values);
+  EXPECT_TRUE(tkrzw_index_remove(index, "single", -1, "1", -1));
+  EXPECT_TRUE(tkrzw_index_remove(index, "double", 5, "11", 2));
+  EXPECT_FALSE(tkrzw_index_remove(index, "triple", -1, "x", -1));
+  values = tkrzw_index_get_values(index, "double", -1, 0, &num_values);
+  ASSERT_NE(nullptr, values);
+  ASSERT_EQ(1, num_values);
+  EXPECT_EQ("22", std::string(values[0].ptr, values[0].size));
+  tkrzw_free_str_array(values, num_values);
+  EXPECT_EQ(3, tkrzw_index_count(index));
+  char* path_ptr = tkrzw_index_get_file_path(index);
+  ASSERT_NE(nullptr, path_ptr);
+  EXPECT_EQ(file_path, path_ptr);
+  free(path_ptr);
+  EXPECT_TRUE(tkrzw_index_synchronize(index, false));
+  EXPECT_TRUE(tkrzw_index_rebuild(index));
+  EXPECT_EQ(3, tkrzw_index_count(index));
+  EXPECT_TRUE(tkrzw_index_clear(index));
+  EXPECT_EQ(0, tkrzw_index_count(index));
+  EXPECT_TRUE(tkrzw_index_is_writable(index));
+  EXPECT_TRUE(tkrzw_index_add(index, "first", -1, "1", -1));
+  EXPECT_TRUE(tkrzw_index_add(index, "second", -1, "2", -1));
+  EXPECT_TRUE(tkrzw_index_add(index, "third", -1, "3", -1));
+  TkrzwIndexIter* iter = tkrzw_index_make_iterator(index);
+  tkrzw_index_iter_first(iter);
+  char* key_ptr = nullptr;
+  int32_t key_size = 0;
+  char* value_ptr = nullptr;
+  int32_t value_size = 0;
+  ASSERT_TRUE(tkrzw_index_iter_get(iter, &key_ptr, &key_size, &value_ptr, &value_size));
+  EXPECT_EQ("first", std::string_view(key_ptr, key_size));
+  EXPECT_EQ("1", std::string_view(value_ptr, value_size));
+  free(value_ptr);
+  free(key_ptr);
+  tkrzw_index_iter_next(iter);
+  ASSERT_TRUE(tkrzw_index_iter_get(iter, &key_ptr, &key_size, &value_ptr, &value_size));
+  EXPECT_EQ("second", std::string_view(key_ptr, key_size));
+  EXPECT_EQ("2", std::string_view(value_ptr, value_size));
+  free(value_ptr);
+  free(key_ptr);
+  tkrzw_index_iter_next(iter);
+  ASSERT_TRUE(tkrzw_index_iter_get(iter, &key_ptr, &key_size, &value_ptr, &value_size));
+  EXPECT_EQ("third", std::string_view(key_ptr, key_size));
+  EXPECT_EQ("3", std::string_view(value_ptr, value_size));
+  free(value_ptr);
+  free(key_ptr);
+  tkrzw_index_iter_next(iter);
+  ASSERT_FALSE(tkrzw_index_iter_get(iter, &key_ptr, &key_size, &value_ptr, &value_size));
+  tkrzw_index_iter_last(iter);
+  ASSERT_TRUE(tkrzw_index_iter_get(iter, &key_ptr, &key_size, &value_ptr, &value_size));
+  EXPECT_EQ("third", std::string_view(key_ptr, key_size));
+  EXPECT_EQ("3", std::string_view(value_ptr, value_size));
+  free(value_ptr);
+  free(key_ptr);
+  tkrzw_index_iter_previous(iter);
+  ASSERT_TRUE(tkrzw_index_iter_get(iter, &key_ptr, &key_size, &value_ptr, &value_size));
+  EXPECT_EQ("second", std::string_view(key_ptr, key_size));
+  EXPECT_EQ("2", std::string_view(value_ptr, value_size));
+  free(value_ptr);
+  free(key_ptr);
+  tkrzw_index_iter_previous(iter);
+  ASSERT_TRUE(tkrzw_index_iter_get(iter, &key_ptr, &key_size, &value_ptr, &value_size));
+  EXPECT_EQ("first", std::string_view(key_ptr, key_size));
+  EXPECT_EQ("1", std::string_view(value_ptr, value_size));
+  free(value_ptr);
+  free(key_ptr);
+  tkrzw_index_iter_previous(iter);
+  ASSERT_FALSE(tkrzw_index_iter_get(iter, &key_ptr, &key_size, &value_ptr, &value_size));
+  tkrzw_index_iter_jump(iter, "second", -1, "", -1);
+  ASSERT_TRUE(tkrzw_index_iter_get(iter, &key_ptr, &key_size, &value_ptr, &value_size));
+  EXPECT_EQ("second", std::string_view(key_ptr, key_size));
+  EXPECT_EQ("2", std::string_view(value_ptr, value_size));
+  free(value_ptr);
+  free(key_ptr);
+  tkrzw_index_iter_free(iter);
+  EXPECT_TRUE(tkrzw_index_close(index));
+}
+
 // END OF FILE
