@@ -18,7 +18,17 @@
 
 namespace tkrzw {
 
-#if defined(_SYS_POSIX_) && !defined(_TKRZW_STDONLY)
+#if defined(_SYS_MACOSX_)
+
+inline void* tkrzw_memmem(const void* haystack, size_t haystacklen,
+                          const void* needle, size_t needlelen) {
+  if (needlelen == 0) {
+    return const_cast<void*>(haystack);
+  }
+  return memmem(haystack, haystacklen, needle, needlelen);
+}
+
+#elif defined(_SYS_POSIX_) && !defined(_TKRZW_STDONLY)
 
 inline void* tkrzw_memmem(const void* haystack, size_t haystacklen,
                           const void* needle, size_t needlelen) {
@@ -343,33 +353,33 @@ void VSPrintF(std::string* dest, const char* format, va_list ap) {
         }
         case 'd': {
           char tbuf[NUM_BUFFER_SIZE];
-          size_t tsiz;
+          size_t tsiz = 0;
           if (lnum >= 2) {
-            tsiz = std::sprintf(tbuf, cbuf, va_arg(ap, long long));
+            tsiz = std::snprintf(tbuf, sizeof(tbuf), cbuf, va_arg(ap, long long));
           } else if (lnum >= 1) {
-            tsiz = std::sprintf(tbuf, cbuf, va_arg(ap, long));
+            tsiz = std::snprintf(tbuf, sizeof(tbuf), cbuf, va_arg(ap, long));
           } else {
-            tsiz = std::sprintf(tbuf, cbuf, va_arg(ap, int));
+            tsiz = std::snprintf(tbuf, sizeof(tbuf), cbuf, va_arg(ap, int));
           }
           dest->append(tbuf, tsiz);
           break;
         }
         case 'o': case 'u': case 'x': case 'X': case 'c': {
           char tbuf[NUM_BUFFER_SIZE];
-          size_t tsiz;
+          size_t tsiz = 0;
           if (lnum >= 2) {
-            tsiz = std::sprintf(tbuf, cbuf, va_arg(ap, unsigned long long));
+            tsiz = std::snprintf(tbuf, sizeof(tbuf), cbuf, va_arg(ap, unsigned long long));
           } else if (lnum >= 1) {
-            tsiz = std::sprintf(tbuf, cbuf, va_arg(ap, unsigned long));
+            tsiz = std::snprintf(tbuf, sizeof(tbuf), cbuf, va_arg(ap, unsigned long));
           } else {
-            tsiz = std::sprintf(tbuf, cbuf, va_arg(ap, unsigned int));
+            tsiz = std::snprintf(tbuf, sizeof(tbuf), cbuf, va_arg(ap, unsigned int));
           }
           dest->append(tbuf, tsiz);
           break;
         }
         case 'e': case 'E': case 'f': case 'g': case 'G': {
           char tbuf[NUM_BUFFER_SIZE * 2];
-          size_t tsiz;
+          size_t tsiz = 0;
           if (lnum >= 1) {
             tsiz = std::snprintf(tbuf, sizeof(tbuf), cbuf, va_arg(ap, long double));
           } else {
@@ -384,7 +394,7 @@ void VSPrintF(std::string* dest, const char* format, va_list ap) {
         }
         case 'p': {
           char tbuf[NUM_BUFFER_SIZE];
-          size_t tsiz = std::sprintf(tbuf, "%p", va_arg(ap, void*));
+          size_t tsiz = std::snprintf(tbuf, sizeof(tbuf), "%p", va_arg(ap, void*));
           dest->append(tbuf, tsiz);
           break;
         }
@@ -402,7 +412,7 @@ void VSPrintF(std::string* dest, const char* format, va_list ap) {
 
 std::string ToString(double data) {
   char buf[NUM_BUFFER_SIZE];
-  int32_t size = std::sprintf(buf, "%.6f", data);
+  int32_t size = std::snprintf(buf, sizeof(buf), "%.6f", data);
   while (size > 0 && buf[size - 1] == '0') {
     buf[size--] = '\0';
   }
@@ -1309,7 +1319,6 @@ int32_t StrWordSearch(std::string_view text, std::string_view pattern) {
       size = text.size() - (rp - text.data());
       continue;
     }
-
     return pv - text.data();
   }
   return -1;
